@@ -248,8 +248,8 @@ class NmfModel:
         if not self.is_ok():
             self.test_model()
         whd = self.get_w() @ self.get_h().T
-        whd.index = self._origina_mat.index
-        whd.columns = self._origin_mat.columns
+        whd.index = self._mat.index
+        whd.columns = self._mat.columns
         return whd
 
     def get_w_hhi_1(self, do_sort=True):
@@ -261,6 +261,21 @@ class NmfModel:
             res = res.sort_values(by=res.columns[0], axis=0, ascending=False)
         return res
 
+    def get_h_hhi_1(self, do_sort=True):
+        if not self.is_ok():
+            self.test_model()
+        h = self.get_h()
+        res = utl.mat_hhi_1(h, axis=1)
+        if do_sort:
+            res = res.sort_values(by=res.columns[0], axis=0, ascending=False)
+        return res
+
+    def get_h_hhi_1_stats(self):
+        df_hhi_1 = self.get_h_hhi_1(do_sort=False)
+        return np.mean(df_hhi_1)
+
+
+
     def get_features_clustering_proba(self):
         if not self.is_ok():
             self.test_model()
@@ -442,9 +457,9 @@ class NmfModel:
             self.test_model()
         return pd.DataFrame(index=self._fnames, columns=["Feature Cluster"], data=self._predictor["HC"])
 
-    def get_features_clusters_hhi(self):
+    def get_features_clusters_hhi_1(self):
         df_feat_cl = self.get_features_clusters()
-        df_count_by_clust = df_feat_cl.groupby("Features Cluster").size().reset_index(name="counts")
+        df_count_by_clust = df_feat_cl.groupby("Feature Cluster").size().reset_index(name="counts")
         return utl.hhi_1(df_count_by_clust["counts"].values)
 
     def sort_observations_by_cluster(self):
@@ -505,16 +520,31 @@ class NmfModel:
                         break
         return sorted_features
 
-    def get_row_proba_scores(self):
+    def get_w_proba_scores(self, axis=1):
+        """
+
+         get_w_proba_scores:
+         Divides each line of W by its mean, transforming it into a proba distribution over the components.
+         Takes the normalized entropy of each line
+         Parameters
+         ----------
+
+         self: the current nmfmodel object
+         axis: indicates wether to take proba
+
+         :returns:
+         a DataFrame containing the features (attributes) as columns, the observations (e.g stocks) as lines
+         """
         w = self.get_w()
-        wm = w.apply(np.nansum, axis=1)
+        wm = w.apply(np.nansum, axis=axis)
         wp = w.div(wm, axis=0)
         nc = self._nfeat
         went = wp.applymap(lambda p: 0 if p == 0.0 else p * math.log(p, 2))
-        return 1 + went.apply(np.nansum, axis=1) / math.log(nc, 2)
+        w_p_scores = 1 + went.apply(np.nansum, axis=1) / math.log(nc, 2)
+        return w_p_scores
 
     def get_specific_cluster_contribution(self):
-        row_scores = self.get_row_proba_scores()
+        row_scores = self.get_w_proba_scores()
         # non standard convention: 0 is the best score (perfect clustering)
         return 1 - np.mean(row_scores)
 
@@ -574,7 +604,7 @@ def compare_nmf_fs_to_random(train_set: Union[np.ndarray, pd.DataFrame],
     :param test_set:
     :param test_labels:
     :param set_name:
-    :param ncomp_range:
+    :param ncomp:
     :param kwargs:
     :return:
     """
@@ -602,6 +632,24 @@ def compare_nmf_fs_to_random(train_set: Union[np.ndarray, pd.DataFrame],
     score_test_reduced = svc_model_reduced.score(df_x_test_reduced, test_labels)
 
 
+def scree_plot(mat: Union[np.ndarray, pd.DataFrame],
+               ncmin: int, ncmax: int):
+    columns = []
+    ncomp = ncmin
+    nmfmodel = NmfModel(mat=mat, ncomp=ncomp, regularization="components")
+    df_scree_plot = pd.DataFrame(columns=columns)
+    for ncomp in range(ncmin, ncmax+1):
+        nmfmodel.set_ncomp(ncomp)
+        line = pd.DataFrame(index=[ncomp], columns=columns)
+        err = line.loc[ncomp, "Error"] = nmfmodel.get_precision()
+        vol = line.loc[ncomp, "Volume"] = nmfmodel.get_volume()
+        line.loc[ncomp, "Err_Vol_Ratio"] = err / vol if vol > 0 else 1e6
+        line.loc[ncomp, "W_entropy"] = nmfmodel.get_specific_cluster_contribution()
+        line.loc[ncomp, "H_clust_hhi_1"] = nmfmodel.get_features_clusters_hhi_1()
+        df_scree_plot = pd.concat([df_scree_plot, line], axis=0)
+    return df_scree_plot
+
+
 
 
 
